import discord
from cpl_core.logging import LoggerABC
from cpl_core.configuration import ConfigurationABC
from gismo_core.abc.bot_service_abc import BotServiceABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC
from modules.permission.configuration.permission_settings import PermissionSettings


class PermissionService(PermissionServiceABC):

    def __init__(self, logger: LoggerABC, bot: BotServiceABC, config: ConfigurationABC):
        PermissionServiceABC.__init__(self)
        self._logger = logger
        self._bot = bot
        self._config = config

        self._admin_role_ids: dict[str, list[int]] = {}
        self._admin_roles: dict[list[discord.Role]] = {}
        self._admins: dict[list[discord.Member]] = {}
        
        self._moderator_role_ids: dict[list[int]] = {}
        self._moderator_roles: dict[list[discord.Role]] = {}
        self._moderators: dict[list[discord.Member]] = {}

    def on_ready(self): 
        for guild in self._bot.guilds:
            guild: discord.Guild = guild

            settings: PermissionSettings = self._config.get_configuration(f'Permission_{guild.id}')
            if settings is None:
                self._logger.error(__name__, 'Permission settings not found')
                return
            
            self._admin_role_ids[guild.id] = settings.admin_roles
            self._admin_roles[guild.id] = []
            self._admins[guild.id] = []
            
            self._moderator_role_ids[guild.id] = settings.moderator_roles
            self._moderator_roles[guild.id] = []
            self._moderators[guild.id] = []

            for role in guild.roles:
                role: discord.Role = role

                if role.id in self._admin_role_ids:
                    self._admin_roles[guild.id].append(role)

                    for member in role.members:
                        self._admins[guild.id].append(member)

                if role.id in self._moderator_role_ids:
                    self._moderator_roles[guild.id].append(role)

                    for member in role.members:
                        self._moderators[guild.id].append(member)

    def on_member_update(self, before: discord.Member, after: discord.Member):
        g_id = after.guild.id
        
        if before in self._admin_roles[g_id] and after not in self._admin_roles[g_id]:
            self._admins[g_id].remove(after)
        
        elif before not in self._admin_roles[g_id] and after in self._admin_roles[g_id]:
            self._admins[g_id].append(after)

        if before in self._moderator_roles[g_id] and after not in self._moderator_roles[g_id]:
            self._moderators[g_id].remove(after)
        
        elif before not in self._moderator_roles[g_id] and after in self._moderator_roles[g_id]:
            self._moderators[g_id].append(after)   

    def get_admin_role_ids(self, g_id: int) -> list[int]:
        return self._admin_role_ids[g_id]

    def get_admin_roles(self, g_id: int) -> list[discord.Role]:
        return self._admin_roles[g_id]

    def get_admins(self, g_id: int) -> list[discord.Member]:
        return self._admins[g_id]

    def get_moderator_role_ids(self, g_id: int) -> list[int]:
        return self._moderator_role_ids[g_id]

    def get_moderator_roles(self, g_id: int) -> list[discord.Role]:
        return self._moderator_roles[g_id]

    def get_moderators(self, g_id: int) -> list[discord.Member]:
        return self._moderators[g_id]
